<?php

namespace Litecms\Bookings\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Bookings\Http\Requests\BookingRequest;
use Litecms\Bookings\Interfaces\BookingRepositoryInterface;
use Litecms\Bookings\Models\Booking;

/**
 * Resource controller class for booking.
 */
class BookingResourceController extends BaseController
{

    /**
     * Initialize booking resource controller.
     *
     * @param type BookingRepositoryInterface $booking
     *
     * @return null
     */
    public function __construct(BookingRepositoryInterface $booking)
    {
        parent::__construct();
        $this->repository = $booking;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Bookings\Repositories\Criteria\BookingResourceCriteria::class);
    }

    /**
     * Display a list of booking.
     *
     * @return Response
     */
    public function index(BookingRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Bookings\Repositories\Presenter\BookingPresenter::class)
                ->$function();
        }

        $bookings = $this->repository->paginate();

        return $this->response->title(trans('bookings::booking.names'))
            ->view('bookings::booking.index', true)
            ->data(compact('bookings', 'view'))
            ->output();
    }

    /**
     * Display booking.
     *
     * @param Request $request
     * @param Model   $booking
     *
     * @return Response
     */
    public function show(BookingRequest $request, Booking $booking)
    {

        if ($booking->exists) {
            $view = 'bookings::booking.show';
        } else {
            $view = 'bookings::booking.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('bookings::booking.name'))
            ->data(compact('booking'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new booking.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BookingRequest $request)
    {

        $booking = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('bookings::booking.name')) 
            ->view('bookings::booking.create', true) 
            ->data(compact('booking'))
            ->output();
    }

    /**
     * Create new booking.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BookingRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $booking                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('bookings::booking.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('bookings/booking/' . $booking->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/bookings/booking'))
                ->redirect();
        }

    }

    /**
     * Show booking for editing.
     *
     * @param Request $request
     * @param Model   $booking
     *
     * @return Response
     */
    public function edit(BookingRequest $request, Booking $booking)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('bookings::booking.name'))
            ->view('bookings::booking.edit', true)
            ->data(compact('booking'))
            ->output();
    }

    /**
     * Update the booking.
     *
     * @param Request $request
     * @param Model   $booking
     *
     * @return Response
     */
    public function update(BookingRequest $request, Booking $booking)
    {
        try {
            $attributes = $request->all();

            $booking->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('bookings::booking.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('bookings/booking/' . $booking->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('bookings/booking/' . $booking->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the booking.
     *
     * @param Model   $booking
     *
     * @return Response
     */
    public function destroy(BookingRequest $request, Booking $booking)
    {
        try {

            $booking->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('bookings::booking.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('bookings/booking/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('bookings/booking/' . $booking->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple booking.
     *
     * @param Model   $booking
     *
     * @return Response
     */
    public function delete(BookingRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('bookings::booking.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('bookings/booking'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/bookings/booking'))
                ->redirect();
        }

    }

    /**
     * Restore deleted bookings.
     *
     * @param Model   $booking
     *
     * @return Response
     */
    public function restore(BookingRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('bookings::booking.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/bookings/booking'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/bookings/booking/'))
                ->redirect();
        }

    }

}
